import warnings
import re

try:
    import markdown
    from magika import Magika
    from bs4 import BeautifulSoup
except ImportError:
    raise ImportError(
        "Required libraries are not installed. Please install 'markdown', 'magika', and 'beautifulsoup4'."
    )


class MarkupRemover:
    def __init__(self):
        self.magic = Magika()

    def __call__(self, text):
        res = self.magic.identify_bytes(bytes(text, encoding="raw_unicode_escape"))
        label = res.output.label

        if label == "markdown":
            html = markdown.markdown(text)
            text = self._html_to_text(html, features="lxml")
        elif label == "html":
            text = self._html_to_text(text, features="lxml")
        elif label in ("xml", "asp"):
            text = self._html_to_text(text, features="xml")
        elif label == "txt":
            pass
        else:
            warnings.warn(
                f"MarkupRemover: Unknown markup type '{label}' detected. "
                "Returning the original text."
            )

        text = text.replace("\n", " ")
        text = text.replace("\r", " ")
        text = text.replace("\t", " ")
        text = re.sub(r"\s\s+", " ", text)
        text = text.strip()

        return text

    def _html_to_text(self, text, features):
        soup = BeautifulSoup(text, features=features)
        return soup.get_text().strip()



from .markup_remover import MarkupRemover


def test_markdown():
    text = """
# Header 1
## Header 2
### Header 3
#### Header 4
##### Header 5
###### Header 6

- List item 1
- List item 2
- List item 3

[Link](https://example.com)
"""
    stripper = MarkupRemover()
    result = stripper(text)

    assert (
        result
        == "Header 1 Header 2 Header 3 Header 4 Header 5 Header 6 List item 1 List item 2 List item 3 Link"
    )


def test_html():
    text = """
    <html>
        <head>
            <title>Title</title>
        </head>
        <body>
            <h1>Header 1</h1>
            <p>Paragraph 1</p>
            <a href="https://example.com">Link</a>
        </body>
    </html>
    """
    stripper = MarkupRemover()
    result = stripper(text)

    assert result == "Title Header 1 Paragraph 1 Link"


def test_xml():
    text = """
    <note>
        <to>Tove</to>
        <from>Jani</from>
        <heading>Reminder</heading>
        <body>Don't forget me this weekend!</body>
    </note>
    """
    stripper = MarkupRemover()
    result = stripper(text)

    assert result == "Tove Jani Reminder Don't forget me this weekend!"


def test_other():
    text = "This is a plain text without any markup."
    stripper = MarkupRemover()
    result = stripper(text)

    assert result == text


def test_empty():
    text = ""
    stripper = MarkupRemover()
    result = stripper(text)

    assert result == ""


def test_whitespace():
    text = "    \n\t Text \r  "
    stripper = MarkupRemover()
    result = stripper(text)

    assert result == "Text"
